@@ -400,7 +400,7 @@ def send_link_rel(request, pagename):
     for fname in files:
         url = getAttachUrl(pagename, fname, request, do='view', escaped=1)
         request.write(u'<link rel="Appendix" title="%s" href="%s">\n' % (
-                      wikiutil.escape(fname), url))
+                      wikiutil.escape(fname, 1), url))
 
 
 def send_hotdraw(pagename, request):
@@ -438,7 +438,7 @@ def send_hotdraw(pagename, request):
     'pngpath': pngpath, 'timestamp': timestamp,
     'pubpath': pubpath, 'drawpath': drawpath,
     'savelink': savelink, 'pagelink': pagelink, 'helplink': helplink,
-    'basename': basename
+    'basename': wikiutil.escape(basename, 1),
 })
 
 
@@ -482,7 +482,7 @@ def send_uploadform(pagename, request):
     'action_name': action_name,
     'upload_label_file': _('File to upload'),
     'upload_label_rename': _('Rename to'),
-    'rename': request.form.get('rename', [''])[0],
+    'rename': wikiutil.escape(request.form.get('rename', [''])[0], 1),
     'upload_label_overwrite': _('Overwrite existing attachment of same name'),
     'overwrite_checked': ('', 'checked')[request.form.get('overwrite', ['0'])[0] == '1'],
     'upload_button': _('Upload'),
