@@ -2,8 +2,8 @@
 """
     MoinMoin - LocalSiteMap action
 
-    The LocalSiteMap action gives you a page that shows 
-    nearby links.  This is an example of what appears on the 
+    The LocalSiteMap action gives you a page that shows
+    nearby links.  This is an example of what appears on the
     page (names are linkable on the real page):
 
     MoinMoin
@@ -36,7 +36,7 @@ def execute(pagename, request):
     # This action generate data using the user language
     request.setContentLanguage(request.lang)
 
-    request.theme.send_title(_('Local Site Map for "%s"', formatted=False) % (pagename), pagename=pagename)
+    request.theme.send_title(_('Local Site Map for "%s"') % (pagename), pagename=pagename)
 
     # Start content - IMPORTANT - witout content div, there is no
     # direction support!
@@ -72,7 +72,7 @@ class LocalSiteMap:
         pg = Page(request, name)
         action = __name__.split('.')[-1]
         self.append('&nbsp;' * (5*depth+1))
-        self.append(pg.link_to(request, wikiutil.escape(name), querystr={'action': action}))
+        self.append(pg.link_to(request, querystr={'action': action}))
         self.append("&nbsp;<small>[")
         self.append(pg.link_to(request, 'view'))
         self.append("</small>]<br>")
