@@ -10,6 +10,9 @@
 
 import re, zipfile
 
+from MoinMoin import log
+logging = log.getLogger(__name__)
+
 rx_stripxml = re.compile("<[^>]*?>", re.DOTALL|re.MULTILINE)
 
 def execute(indexobj, filename):
@@ -19,7 +22,7 @@ def execute(indexobj, filename):
         zf.close()
         data = " ".join(rx_stripxml.sub(" ", data).split())
     except RuntimeError, err:
-        indexobj.request.log(str(err))
+        logging.error(str(err))
         data = ""
     return data.decode('utf-8')
 
