@@ -4,12 +4,15 @@
 
     Depends on: nothing (only python with zlib)
 
-    @copyright: 2006 by ThomasWaldmann MoinMoin:ThomasWaldmann
+    @copyright: 2006 MoinMoin:ThomasWaldmann
     @license: GNU GPL, see COPYING for details.
 """
 
 import re, zipfile
 
+from MoinMoin import log
+logging = log.getLogger(__name__)
+
 rx_stripxml = re.compile("<[^>]*?>", re.DOTALL|re.MULTILINE)
 
 def execute(indexobj, filename):
@@ -19,7 +22,7 @@ def execute(indexobj, filename):
         zf.close()
         data = " ".join(rx_stripxml.sub(" ", data).split())
     except RuntimeError, err:
-        indexobj.request.log(str(err))
+        logging.error(str(err))
         data = ""
     return data.decode('utf-8')
 
