@@ -7,11 +7,18 @@
 """
 import os.path, re
 
+from MoinMoin import log
+logging = log.getLogger(__name__)
+
 from MoinMoin.formatter import FormatterBase
 from MoinMoin import wikiutil, i18n
 from MoinMoin.Page import Page
 from MoinMoin.action import AttachFile
-from MoinMoin.support.python_compatibility import set, rsplit
+from MoinMoin.support.python_compatibility import set
+
+# insert IDs into output wherever they occur
+# warning: breaks toggle line numbers javascript
+_id_debug = False
 
 line_anchors = True
 prettyprint = False
@@ -175,24 +182,14 @@ class Formatter(FormatterBase):
 
     def __init__(self, request, **kw):
         FormatterBase.__init__(self, request, **kw)
-
-        # inline tags stack. When an inline tag is called, it goes into
-        # the stack. When a block element starts, all inline tags in
-        # the stack are closed.
-        self._inlineStack = []
-
-        # stack of all tags
-        self._tag_stack = []
         self._indent_level = 0
 
         self._in_code = 0 # used by text_gedit
         self._in_code_area = 0
         self._in_code_line = 0
-        self._code_area_num = 0
         self._code_area_js = 0
         self._code_area_state = ['', 0, -1, -1, 0]
         self._show_section_numbers = None
-        self._content_ids = []
         self.pagelink_preclosed = False
         self._is_included = kw.get('is_included', False)
         self.request = request
@@ -248,7 +245,7 @@ class Formatter(FormatterBase):
 
         Setting a value to None rather than a string (or string
         coercible) will remove that attribute from the list.
-        
+
         If the list of allowed_attrs is provided, then an error is
         raised if an HTML attribute is encountered that is not in that
         list (or is not a common attribute which is always allowed or
@@ -320,13 +317,15 @@ class Formatter(FormatterBase):
             return ' '.join(all)
         return ''
 
-    def _open(self, tag, newline=False, attr=None, allowed_attrs=None, **kw):
+    def _open(self, tag, newline=False, attr=None, allowed_attrs=None,
+              is_unique=False, **kw):
         """ Open a tag with optional attributes (INTERNAL USE BY HTML FORMATTER ONLY!)
-        
+
         @param tag: html tag, string
         @param newline: render tag so following data is on a separate line
         @param attr: dict with tag attributes
         @param allowed_attrs: list of allowed attributes for this element
+        @param is_unique: ID is already unique
         @param kw: arbitrary attributes and values
         @rtype: string ?
         @return: open tag with attributes as a string
@@ -334,6 +333,23 @@ class Formatter(FormatterBase):
         # If it is self-closing, then don't expect a closing tag later on.
         is_self_closing = (tag in _self_closing_tags) and ' /' or ''
 
+        # make ID unique
+        id = None
+        if not is_unique:
+            if attr and 'id' in attr:
+                id = self.make_id_unique(attr['id'])
+                id = self.qualify_id(id)
+                attr['id'] = id
+            if 'id' in kw:
+                id = self.make_id_unique(kw['id'])
+                id = self.qualify_id(id)
+                kw['id'] = id
+        else:
+            if attr and 'id' in attr:
+                id = attr['id']
+            if 'id' in kw:
+                id = kw['id']
+
         if tag in _blocks:
             # Block elements
             result = []
@@ -348,22 +364,14 @@ class Formatter(FormatterBase):
             result.append('<%s%s%s>' % (tag, attributes, is_self_closing))
             if newline:
                 result.append(self._newline())
+            if _id_debug and id:
+                result.append('(%s) ' % id)
             tagstr = ''.join(result)
         else:
             # Inline elements
-            # Add to inlineStack
-            if not is_self_closing:
-                # Only push on stack if we expect a close-tag later
-                self._inlineStack.append(tag)
-            # Format
             tagstr = '<%s%s%s>' % (tag,
                                       self._formatAttributes(attr, allowed_attrs, **kw),
                                       is_self_closing)
-        # XXX SENSE ???
-        #if not self.close:
-        #    self._tag_stack.append(tag)
-        #    if tag in _indenting_tags:
-        #        self._indent_level += 1
         return tagstr
 
     def _close(self, tag, newline=False):
@@ -379,32 +387,15 @@ class Formatter(FormatterBase):
             tagstr = ''
         elif tag in _blocks:
             # Block elements
-            # Close all tags in inline stack
-            # Work on a copy, because close(inline) manipulate the stack
             result = []
-            stack = self._inlineStack[:]
-            stack.reverse()
-            for inline in stack:
-                result.append(self._close(inline))
-            # Format with newline
             if newline:
                 result.append(self._newline())
             result.append('</%s>' % (tag))
             tagstr = ''.join(result)
         else:
-            # Inline elements 
-            # Pull from stack, ignore order, that is not our problem.
-            # The code that calls us should keep correct calling order.
-            if tag in self._inlineStack:
-                self._inlineStack.remove(tag)
+            # Inline elements
             tagstr = '</%s>' % tag
 
-        # XXX see other place marked with "SENSE"
-        #if tag in _self_closing_tags:
-        #    self._tag_stack.pop()
-        #    if tag in _indenting_tags:
-        #        # decrease indent level
-        #        self._indent_level -= 1
         if newline:
             tagstr += self._newline()
         return tagstr
@@ -415,23 +406,19 @@ class Formatter(FormatterBase):
         """ Start page content div.
 
         A link anchor is provided at the beginning of the div, with
-        an id of 'top' or 'top_xxxx' if the content_id argument is
-        set to 'xxxx'.
+        an id of 'top' (modified by the request ID cache).
         """
 
-        # Setup id
-        if content_id != 'content':
-            aid = 'top_%s' % (content_id,)
-        else:
-            aid = 'top'
-        self._content_ids.append(content_id)
+        if hasattr(self, 'page'):
+            self.request.begin_include(self.page.page_name)
+
         result = []
         # Use the content language
         attr = self._langAttr(self.request.content_lang)
         attr['id'] = content_id
         result.append(self._open('div', newline=False, attr=attr,
                                  allowed_attrs=['align'], **kw))
-        result.append(self.anchordef(aid))
+        result.append(self.anchordef('top'))
         if newline:
             result.append('\n')
         return ''.join(result)
@@ -440,30 +427,21 @@ class Formatter(FormatterBase):
         """ Close page content div.
 
         A link anchor is provided at the end of the div, with
-        an id of 'bottom' or 'bottom_xxxx' if the content_id argument
-        to the previus startContent() call was set to 'xxxx'.
+        an id of 'bottom' (modified by the request ID cache).
         """
 
-        # Setup id
-        try:
-            cid = self._content_ids.pop()
-        except:
-            cid = 'content'
-        if cid != 'content':
-            aid = 'bottom_%s' % (cid,)
-        else:
-            aid = 'bottom'
-
         result = []
-        result.append(self.anchordef(aid))
+        result.append(self.anchordef('bottom'))
         result.append(self._close('div', newline=newline))
+        if hasattr(self, 'page'):
+            self.request.end_include()
         return ''.join(result)
 
     def lang(self, on, lang_name):
         """ Insert text with specific lang and direction.
-        
+
             Enclose within span tag if lang_name is different from
-            the current lang    
+            the current lang
         """
         tag = 'span'
         if lang_name != self.request.current_lang:
@@ -510,8 +488,6 @@ class Formatter(FormatterBase):
         wikitag, wikiurl, wikitail, wikitag_bad = wikiutil.resolve_interwiki(self.request, interwiki, pagename)
         wikiurl = wikiutil.mapURL(self.request, wikiurl)
         if wikitag == 'Self': # for own wiki, do simple links
-            if '#' in wikitail:
-                wikitail, kw['anchor'] = rsplit(wikitail, '#', 1)
             wikitail = wikiutil.url_unquote(wikitail)
             try: # XXX this is the only place where we access self.page - do we need it? Crashes silently on actions!
                 pagename = wikiutil.AbsPageName(self.page.page_name, wikitail)
@@ -524,6 +500,9 @@ class Formatter(FormatterBase):
                 if querystr:
                     separator = ('?', '&')['?' in href]
                     href = '%s%s%s' % (href, separator, wikiutil.makeQueryString(querystr))
+                anchor = kw.get('anchor')
+                if anchor:
+                    href = '%s#%s' % (href, self.sanitize_to_id(anchor))
                 if wikitag_bad:
                     html_class = 'badinterwiki'
                 else:
@@ -533,28 +512,23 @@ class Formatter(FormatterBase):
             else:
                 return self.url(0)
 
-    def url(self, on, url=None, css=None, do_escape=0, **kw):
+    def url(self, on, url=None, css=None, do_escape=None, **kw):
         """
-        Inserts an <a> element.
+        Inserts an <a> element (you can give any A tag attributes as kw args).
 
-        Call once with on=1 to start the link, and again with on=0
-        to end it (no other arguments are needed when on==0).
-
-        do_escape: filters url through wikiutil.escape
-
-        Keyword params:
-            url - the URL to link to; will go through Wiki URL mapping.
-            css - a space-separated list of CSS classes
-            attrs -  just include this string verbatim inside
-                     the <a> element; can be used for arbitrary attrs;
-                     all escaping and quoting is the caller's responsibility.
-
-        Note that the 'attrs' keyword argument is for backwards compatibility
-        only.  It should not be used for new code -- instead just pass
-        any attributes in as separate keyword arguments.
-
-        1.5.3: removed ugly "attrs" keyword argument handling code
+        @param on: 1 to start the link, 0 to end the link (no other arguments are needed when on==0).
+        @param url: the URL to link to; will go through Wiki URL mapping.
+        @param css: a space-separated list of CSS classes
+        @param do_escape: DEPRECATED and not used any more, please remove it from your code!
+                          We will remove this parameter in moin 1.8 (it used to filter url
+                          param through wikiutil.escape, but text_html formatter's _open
+                          will do it again, so this just leads to double escaping now).
         """
+        if do_escape is not None:
+            if do_escape:
+                logging.warning("Deprecation warning: MoinMoin.formatter.text_html.url being called with do_escape=1/True parameter, please review caller.")
+            else:
+                logging.warning("Deprecation warning: MoinMoin.formatter.text_html.url being called with do_escape=0/False parameter, please remove it from the caller.")
         if on:
             attrs = self._langAttr()
 
@@ -564,8 +538,6 @@ class Formatter(FormatterBase):
                 del kw['href']
             if url is not None:
                 url = wikiutil.mapURL(self.request, url)
-                if do_escape:
-                    url = wikiutil.escape(url)
                 attrs['href'] = url
 
             if css:
@@ -576,7 +548,6 @@ class Formatter(FormatterBase):
             markup = self._close('a')
         return markup
 
-
     def anchordef(self, id):
         """Inserts an invisible element used as a link target.
 
@@ -587,7 +558,9 @@ class Formatter(FormatterBase):
         # Don't add newlines, \n, as it will break pre and
         # line-numbered code sections (from line_achordef() method).
         #return '<a id="%s"></a>' % (id, ) # do not use - this breaks PRE sections for IE
-        return '<span class="anchor" id="%s"></span>' % wikiutil.escape(id, 1)
+        id = self.make_id_unique(id)
+        id = self.qualify_id(id)
+        return '<span class="anchor" id="%s"></span>' % id
 
     def line_anchordef(self, lineno):
         if line_anchors:
@@ -609,10 +582,10 @@ class Formatter(FormatterBase):
         The id argument, if provided, is instead the id of this link
         itself and not of the target element the link references.
         """
-
         attrs = self._langAttr()
         if name:
-            attrs['href'] = '#%s' % name
+            name = self.sanitize_to_id(name)
+            attrs['href'] = '#' + self.qualify_id(name)
         if 'href' in kw:
             del kw['href']
         if on:
@@ -643,15 +616,15 @@ class Formatter(FormatterBase):
             querystr['do'] = 'view'
         if on:
             pagename, filename = AttachFile.absoluteName(url, self.page.page_name)
-            #self.request.log("attachment_link: url %s pagename %s filename %s" % (url, pagename, filename))
+            #logging.debug("attachment_link: url %s pagename %s filename %s" % (url, pagename, filename))
             fname = wikiutil.taintfilename(filename)
             if AttachFile.exists(self.request, pagename, fname):
                 target = AttachFile.getAttachUrl(pagename, fname, self.request, do=querystr['do'])
                 title = "attachment:%s" % url
                 css = 'attachment'
             else:
-                target = AttachFile.getAttachUploadUrl(pagename, fname, self.request)
-                title = _('Upload new attachment "%(filename)s"', formatted=False) % {'filename': wikiutil.escape(fname)}
+                target = AttachFile.getAttachUrl(pagename, fname, self.request, upload=True)
+                title = _('Upload new attachment "%(filename)s"') % {'filename': fname}
                 css = 'attachment nonexistent'
             return self.url(on, target, css=css, title=title)
         else:
@@ -665,7 +638,7 @@ class Formatter(FormatterBase):
         if exists:
             kw['css'] = 'attachment'
             kw['src'] = AttachFile.getAttachUrl(pagename, filename, self.request, addts=1)
-            title = _('Inlined image: %(url)s', formatted=False) % {'url': self.text(url)}
+            title = _('Inlined image: %(url)s') % {'url': self.text(url)}
             if not 'title' in kw:
                 kw['title'] = title
             # alt is required for images:
@@ -673,20 +646,14 @@ class Formatter(FormatterBase):
                 kw['alt'] = kw['title']
             return self.image(**kw)
         else:
-            title = _('Upload new attachment "%(filename)s"', formatted=False) % {
-                      'filename': wikiutil.escape(fname)}
+            title = _('Upload new attachment "%(filename)s"') % {'filename': fname}
             img = self.icon('attachimg')
             css = 'nonexistent'
-            return wikiutil.link_tag(
-                self.request,
-                ('%s?action=AttachFile&rename=%s' %
-                 (wikiutil.quoteWikinameURL(pagename),
-                  wikiutil.url_quote_plus(fname))),
-                img,
-                title=title,
-                css_class=css)
+            target = AttachFile.getAttachUrl(pagename, fname, self.request, upload=True)
+            return self.url(1, target, css=css, title=title) + img + self.url(0)
 
     def attachment_drawing(self, url, text, **kw):
+        # XXX text arg is unused!
         _ = self.request.getText
         pagename, filename = AttachFile.absoluteName(url, self.page.page_name)
         fname = wikiutil.taintfilename(filename)
@@ -703,56 +670,44 @@ class Formatter(FormatterBase):
                 fname, filename = gfname, gfilename
 
         # check whether attachment exists, possibly point to upload form
+        drawing_url = AttachFile.getAttachUrl(pagename, fname, self.request, drawing=drawing, upload=True)
         if not exists:
-            linktext = _('Create new drawing "%(filename)s (opens in new window)"', formatted=False)
-            return wikiutil.link_tag(
-                self.request,
-                ('%s?action=AttachFile&rename=%s%s' %
-                 (wikiutil.quoteWikinameURL(pagename),
-                  wikiutil.url_quote_plus(fname),
-                  drawing and ('&drawing=%s' % wikiutil.url_quote(drawing)) or '')),
-                linktext % {'filename': self.text(fname)})
+            title = _('Create new drawing "%(filename)s (opens in new window)"') % {'filename': fname}
+            img = self.icon('attachimg')  # TODO: we need a new "drawimg" in similar grey style and size
+            css = 'nonexistent'
+            return self.url(1, drawing_url, css=css, title=title) + img + self.url(0)
+
+        title = _('Edit drawing %(filename)s (opens in new window)') % {'filename': self.text(fname)}
+        kw['src'] = AttachFile.getAttachUrl(pagename, filename, self.request, addts=1)
+        kw['css'] = 'drawing'
 
         mappath = AttachFile.getFilename(self.request, pagename, drawing + u'.map')
-        edit_link = ('%s?action=AttachFile&rename=%s&drawing=%s' % (
-            wikiutil.quoteWikinameURL(pagename),
-            wikiutil.url_quote_plus(fname),
-            wikiutil.url_quote(drawing)))
-
-        # check for map file
-        if os.path.exists(mappath):
+        try:
+            map = file(mappath, 'r').read()
+        except (IOError, OSError):
+            map = ''
+        if map:
             # we have a image map. inline it and add a map ref to the img tag
-            try:
-                map = file(mappath, 'r').read()
-            except IOError:
-                pass
-            except OSError:
-                pass
-            else:
-                mapid = 'ImageMapOf' + drawing
-                # replace MAPNAME
-                map = map.replace('%MAPNAME%', mapid)
-                # add alt and title tags to areas
-                map = re.sub('href\s*=\s*"((?!%TWIKIDRAW%).+?)"', r'href="\1" alt="\1" title="\1"', map)
-                # add in edit links plus alt and title attributes
-                alt = title = _('Edit drawing %(filename)s (opens in new window)', formatted=False) % {'filename': self.text(fname)}
-                map = map.replace('%TWIKIDRAW%"', '%s" alt="%s" title="%s"' % (edit_link, alt, title))
-                # unxml, because 4.01 concrete will not validate />
-                map = map.replace('/>', '>')
-                alt = title = _('Clickable drawing: %(filename)s', formatted=False) % {'filename': self.text(fname)}
-                return (map + self.image(
-                    alt=alt,
-                    title=title,
-                    src=AttachFile.getAttachUrl(pagename, filename, self.request, addts=1),
-                    usemap='#'+mapid, css="drawing"))
+            mapid = 'ImageMapOf' + drawing
+            map = map.replace('%MAPNAME%', mapid)
+            # add alt and title tags to areas
+            map = re.sub('href\s*=\s*"((?!%TWIKIDRAW%).+?)"', r'href="\1" alt="\1" title="\1"', map)
+            map = map.replace('%TWIKIDRAW%"', '%s" alt="%s" title="%s"' % (drawing_url, title, title))
+            # unxml, because 4.01 concrete will not validate />
+            map = map.replace('/>', '>')
+            title = _('Clickable drawing: %(filename)s') % {'filename': self.text(fname)}
+            if 'title' not in kw:
+                kw['title'] = title
+            if 'alt' not in kw:
+                kw['alt'] = kw['title']
+            kw['usemap'] = '#'+mapid
+            return map + self.image(**kw)
         else:
-            alt = title = _('Edit drawing %(filename)s (opens in new window)', formatted=False) % {'filename': self.text(fname)}
-            return wikiutil.link_tag(self.request,
-                                     edit_link,
-                                     self.image(alt=alt,
-                                                title=title,
-                                                src=AttachFile.getAttachUrl(pagename, filename, self.request, addts=1), css="drawing"),
-                                     title=title)
+            if 'title' not in kw:
+                kw['title'] = title
+            if 'alt' not in kw:
+                kw['alt'] = kw['title']
+            return self.url(1, drawing_url) + self.image(**kw) + self.url(0)
 
 
     # Text ##############################################################
@@ -854,7 +809,6 @@ class Formatter(FormatterBase):
         non-break spaces.
         """
         tag = 'tt'
-        # Maybe we don't need this, because we have tt will be in inlineStack.
         self._in_code = on
         if on:
             return self._open(tag, allowed_attrs=[], **kw)
@@ -966,12 +920,15 @@ function togglenumber(did, nstart, nstep) {
         """
         _ = self.request.getText
         res = []
-        ci = self.request.makeUniqueID('CA-%s_%03d' % (code_id, self._code_area_num))
         if on:
+            code_id = self.sanitize_to_id('CA-%s' % code_id)
+            ci = self.qualify_id(self.make_id_unique(code_id))
+
             # Open a code area
             self._in_code_area = 1
             self._in_code_line = 0
-            self._code_area_state = [ci, show, start, step, start]
+            # id in here no longer used
+            self._code_area_state = [None, show, start, step, start]
 
             # Open the code div - using left to right always!
             attr = {'class': 'codearea', 'lang': 'en', 'dir': 'ltr'}
@@ -989,13 +946,13 @@ function togglenumber(did, nstart, nstep) {
 document.write('<a href="#" onclick="return togglenumber(\'%s\', %d, %d);" \
                 class="codenumbers">%s<\/a>');
 </script>
-''' % (self._code_area_state[0], self._code_area_state[2], self._code_area_state[3],
-       _("Toggle line numbers", formatted=False))
+''' % (ci, self._code_area_state[2], self._code_area_state[3],
+       _("Toggle line numbers"))
                 res.append(toggleLineNumbersLink)
 
             # Open pre - using left to right always!
-            attr = {'id': self._code_area_state[0], 'lang': 'en', 'dir': 'ltr'}
-            res.append(self._open('pre', newline=True, attr=attr))
+            attr = {'id': ci, 'lang': 'en', 'dir': 'ltr'}
+            res.append(self._open('pre', newline=True, attr=attr, is_unique=True))
         else:
             # Close code area
             res = []
@@ -1006,7 +963,6 @@ document.write('<a href="#" onclick="return togglenumber(\'%s\', %d, %d);" \
 
             # Update state
             self._in_code_area = 0
-            self._code_area_num += 1
 
         return ''.join(res)
 
@@ -1045,17 +1001,17 @@ document.write('<a href="#" onclick="return togglenumber(\'%s\', %d, %d);" \
 
     def linebreak(self, preformatted=1):
         """Creates a line break in the HTML output.
-        
+
         If preformatted is true a <br> element is inserted, otherwise
         the linebreak will only be visible in the HTML source.
         """
         if self._in_code_area:
             preformatted = 1
-        return ['\n', '<br />\n'][not preformatted] + self._indent_spaces()
+        return ['\n', '<br>\n'][not preformatted] + self._indent_spaces()
 
     def paragraph(self, on, **kw):
         """Creates a paragraph with a <p> element.
-        
+
         Call once with on=1 to start the region, and a second time
         with on=0 to end it.
         """
@@ -1262,7 +1218,7 @@ document.write('<a href="#" onclick="return togglenumber(\'%s\', %d, %d);" \
     _allowed_table_attrs = {
         'table': ['class', 'id', 'style'],
         'row': ['class', 'id', 'style'],
-        '': ['colspan', 'rowspan', 'class', 'id', 'style'],
+        '': ['colspan', 'rowspan', 'class', 'id', 'style', 'abbr'],
     }
 
     def _checkTableAttr(self, attrs, prefix):
@@ -1312,7 +1268,7 @@ document.write('<a href="#" onclick="return togglenumber(\'%s\', %d, %d);" \
                 pass
         else:
             result['style'] = st
-        #self.request.log("_checkTableAttr returns %r" % result)
+        #logging.debug("_checkTableAttr returns %r" % result)
         return result
 
 
@@ -1424,3 +1380,6 @@ document.write('<a href="#" onclick="return togglenumber(\'%s\', %d, %d);" \
             return self._open(tag, **kw)
         return self._close(tag)
 
+    def sanitize_to_id(self, text):
+        return wikiutil.anchor_name_from_text(text)
+
