@@ -82,6 +82,7 @@ def _check(request, pagename, username, right):
         allowed = acl.may(request, username, right)
         if allowed is not None:
             return allowed
+
     allowed = cache.acl_rights_after.may(request, username, right)
     if allowed is not None:
         return allowed
@@ -131,7 +132,7 @@ class Permissions:
 
     def __getattr__(self, attr):
         """ Shortcut to export getPermission function for all known ACL rights
-        
+
         if attr is one of the rights in acl_rights_valid, then return a
         checking function for it. Else raise an AttributeError.
 
@@ -155,7 +156,7 @@ class AccessControlList:
     Control who may do what on or with a wiki page.
 
     Syntax of an ACL string:
-    
+
         [+|-]User[,User,...]:[right[,right,...]] [[+|-]SomeGroup:...] ...
         ... [[+|-]Known:...] [[+|-]All:...]
 
@@ -195,9 +196,9 @@ class AccessControlList:
         right list are automatically set to NO.
 
     Using Prefixes
-        
+
         To make the system more flexible, there are also two modifiers:
-        the prefixes "+" and "-". 
+        the prefixes "+" and "-".
 
             +SomeUser:read -OtherUser:write
 
@@ -207,13 +208,13 @@ class AccessControlList:
         above acl line does not define if he can or can not write. He
         will be able to write if acl_rights_before or acl_rights_after
         allow this (see configuration options).
-        
+
         Using prefixes, this acl line:
-        
+
             SomeUser:read,write SomeGroup:read,write,admin All:read
 
         Can be written as:
-        
+
             -SomeUser:admin SomeGroup:read,write,admin All:read
 
         Or even:
@@ -225,7 +226,7 @@ class AccessControlList:
         useful on the moin configuration entries though.
 
    Configuration options
-   
+
        cfg.acl_rights_default
            It is is ONLY used when no other ACLs are given.
            Default: "Known:read,write,delete All:read,write",
@@ -239,7 +240,7 @@ class AccessControlList:
            When the page has ACL entries, this will be inserted AFTER
            any page entries.
            Default: ""
-       
+
        cfg.acl_rights_valid
            These are the acceptable (known) rights (and the place to
            extend, if necessary).
@@ -307,7 +308,7 @@ class AccessControlList:
         else: # we have a #acl on the page (self.acl can be [] if #acl is empty!)
             acl = self.acl
         is_group_member = request.dicts.has_member
-        group_re = request.cfg.cache.page_group_regex
+        group_re = request.cfg.cache.page_group_regexact
         allowed = None
         for entry, rightsdict in acl:
             if entry in self.special_users:
@@ -349,11 +350,13 @@ class AccessControlList:
         return None
 
     def _special_Trusted(self, request, name, dowhat, rightsdict):
-        """ check if user <name> is known AND even has logged in using a password.
-            does not work for subsription emails that should be sent to <user>,
+        """ check if user <name> is known AND has logged in using a trusted
+            authentication method.
+            Does not work for subsription emails that should be sent to <user>,
             as he is not logged in in that case.
         """
-        if request.user.trusted and name == request.user.name:
+        if (request.user.name == name and
+            request.user.auth_method in request.cfg.auth_methods_trusted):
             return rightsdict.get(dowhat)
         return None
 
