@@ -67,7 +67,9 @@ class SimpleServer(BaseHTTPServer.HTTPServer):
 
     def server_activate(self):
         BaseHTTPServer.HTTPServer.server_activate(self)
-        logging.info("Serving on %s:%d" % self.server_address)
+        logging.info("%s serving on %s:%d" % (self.__class__.__name__,
+                                              self.server_address[0],
+                                              self.server_address[1]))
 
     def serve_forever(self):
         """Handle one request at a time until we die """
@@ -133,6 +135,7 @@ class ThreadingServer(SimpleServer):
                 return
             t = Thread(target=self.process_request_thread,
                        args=(request, client_address))
+            t.setDaemon(True)
             t.start()
         finally:
             self.lock.release()
@@ -164,11 +167,6 @@ class ThreadPoolServer(SimpleServer):
 
     This server is 5 times faster than ThreadingServer for static
     files, and about the same for wiki pages.
-
-    TODO: sometimes the server won't exit on Conrol-C, and continue to
-    run with few threads (you can kill it with kill -9). Same problem
-    exist with the twisted server. When the problem is finally solved,
-    remove the commented debug prints.
     """
     use_threads = True
 
@@ -186,6 +184,7 @@ class ThreadPoolServer(SimpleServer):
         from threading import Thread
         for dummy in range(self.poolSize):
             t = Thread(target=self.serve_forever_thread)
+            t.setDaemon(True)
             t.start()
         SimpleServer.serve_forever(self)
 
@@ -219,7 +218,6 @@ class ThreadPoolServer(SimpleServer):
             except:
                 self.handle_error(request, client_address)
             self.close_request(request)
-        # sys.stderr.write('thread exiting...\n')
 
     def pop_request(self):
         """ Pop a request from the queue
@@ -240,7 +238,6 @@ class ThreadPoolServer(SimpleServer):
                     self.lock.wait()
         finally:
             self.lock.release()
-        # sys.stderr.write('thread exiting...\n')
         sys.exit()
 
     def die(self):
@@ -256,7 +253,6 @@ class ThreadPoolServer(SimpleServer):
     def wake_all_threads(self):
         self.lock.acquire()
         try:
-            # sys.stderr.write('waking up all threads...\n')
             self.lock.notifyAll()
         finally:
             self.lock.release()
