@@ -38,7 +38,7 @@ class Config(DefaultConfig):
     # Wiki logo. You can use an image, text or both. [Unicode]
     # For no logo or text, use '' - the default is to show the sitename.
     # See also url_prefix setting below!
-    logo_string = u'<img src="/moin_static171/common/moinmoin.png" alt="MoinMoin Logo">'
+    logo_string = u'<img src="/moin_static182/common/moinmoin.png" alt="MoinMoin Logo">'
 
     # name of entry page / front page [Unicode], choose one of those:
 
@@ -49,7 +49,7 @@ class Config(DefaultConfig):
     #page_front_page = u"FrontPage"
 
     # The interwiki name used in interwiki links
-    #interwikiname = 'UntitledWiki'
+    #interwikiname = u'UntitledWiki'
     # Show the interwiki name (and link it to page_front_page) in the Theme,
     # nice for farm setups or when your logo does not show the wiki's name.
     #show_interwiki = 1
@@ -76,12 +76,12 @@ class Config(DefaultConfig):
 
     # The URL prefix we use to access the static stuff (img, css, js).
     # NOT touching this is maybe the best way to handle this setting as moin
-    # uses a good internal default (something like '/moin_static171' for moin
-    # version 1.7.1).
+    # uses a good internal default (something like '/moin_static182' for moin
+    # version 1.8.2).
     # For Twisted and standalone server, the default will automatically work.
     # For others, you should make a matching server config (e.g. an Apache
     # Alias definition pointing to the directory with the static stuff).
-    #url_prefix_static = '/moin_static171'
+    #url_prefix_static = '/moin_static182'
 
 
     # Security ----------------------------------------------------------
@@ -153,7 +153,7 @@ class Config(DefaultConfig):
     # e.g. CategoryFoo -> group 'all' ==  CategoryFoo, group 'key' == Foo
     # moin's code will add ^ / $ at beginning / end when needed
     # You must use Unicode strings here [Unicode]
-    page_category_regex = ur'(?P<all>Category(?P<key>\S+))'
+    page_category_regex = ur'(?P<all>Category(?P<key>(?!Template)\S+))'
     page_dict_regex = ur'(?P<all>(?P<key>\S+)Dict)'
     page_group_regex = ur'(?P<all>(?P<key>\S+)Group)'
     page_template_regex = ur'(?P<all>(?P<key>\S+)Template)'
